import { Container } from 'inversify';
import { IViewport } from '../camera/ICameraService';
export declare type Point = [number, number];
export declare type Bounds = [[number, number], [number, number]];
export interface ILngLat {
    lng: number;
    lat: number;
}
export interface IPoint {
    x: number;
    y: number;
}
export interface IMapWrapper {
    setContainer(container: Container, id: string): void;
}
export interface IMapService<RawMap = {}> {
    map: RawMap;
    init(): void;
    destroy(): void;
    onCameraChanged(callback: (viewport: IViewport) => void): void;
    addMarkerContainer(): void;
    getMarkerContainer(): HTMLElement;
    on(type: string, handler: (...args: any[]) => void): void;
    off(type: string, handler: (...args: any[]) => void): void;
    once(type: string, handler: (...args: any[]) => void): void;
    getContainer(): HTMLElement | null;
    getSize(): [number, number];
    getMinZoom(): number;
    getMaxZoom(): number;
    getType(): string;
    getZoom(): number;
    getCenter(): ILngLat;
    getPitch(): number;
    getRotation(): number;
    getBounds(): Bounds;
    getMapContainer(): HTMLElement | null;
    setRotation(rotation: number): void;
    zoomIn(): void;
    zoomOut(): void;
    panTo(p: Point): void;
    panBy(pixel: Point): void;
    fitBounds(bound: Bounds): void;
    setZoomAndCenter(zoom: number, center: Point): void;
    setZoom(zoom: number): void;
    setMapStyle(style: string): void;
    pixelToLngLat(pixel: Point): ILngLat;
    lngLatToPixel(lnglat: Point): IPoint;
    containerToLngLat(pixel: Point): ILngLat;
    lngLatToContainer(lnglat: Point): IPoint;
}
export declare const MapServiceEvent: string[];
/**
 * 地图初始化配置项
 */
export interface IMapConfig<RawMap = {}> {
    /**
     * 地图实例
     */
    mapInstance?: RawMap;
    /**
     * 容器 DOM id
     */
    id: string | HTMLDivElement;
    /**
     * 地图
     */
    token?: string;
    /**
     * 中心点
     */
    center?: [number, number];
    /**
     * 地图倾角
     */
    pitch?: number;
    /**
     * 地图旋转角度
     */
    rotation?: number;
    /**
     * 缩放等级
     */
    zoom?: number;
    /**
     * 底图样式
     */
    style?: string;
    /**
     * 最小缩放等级
     */
    minZoom?: number;
    /**
     * 最大缩放等级
     */
    maxZoom?: number;
    attributionControl?: boolean;
}
/**
 * 地图相机参数
 * @see
 */
export interface IMapCamera {
    aspect: number;
    fov: number;
    near: number;
    far: number;
    viewportWidth: number;
    viewportHeight: number;
    pitch: number;
    bearing: number;
    zoom: number;
    center: [number, number];
    cameraHeight: number;
    offsetOrigin: [number, number];
}
